"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractCache = exports.DEFAULT_CACHE_ERROR_HANDLER = exports.DEFAULT_LOAD_ERROR_HANDLER = void 0;
const memory_1 = require("./memory");
const Logger_1 = require("./util/Logger");
const InMemoryGroupCache_1 = require("./memory/InMemoryGroupCache");
const DEFAULT_LOAD_ERROR_HANDLER = (err, key, loader, logger) => {
    logger.error(`Error while loading "${key}" with ${loader.name}: ${err.message}`);
};
exports.DEFAULT_LOAD_ERROR_HANDLER = DEFAULT_LOAD_ERROR_HANDLER;
const DEFAULT_CACHE_ERROR_HANDLER = (err, key, cache, logger) => {
    logger.error(`Error while caching "${key}" with ${cache.name}: ${err.message}`);
};
exports.DEFAULT_CACHE_ERROR_HANDLER = DEFAULT_CACHE_ERROR_HANDLER;
class AbstractCache {
    inMemoryCache;
    asyncCache;
    logger;
    cacheUpdateErrorHandler;
    loadErrorHandler;
    runningLoads;
    notificationConsumer;
    notificationPublisher;
    initPromises;
    constructor(config) {
        this.initPromises = [];
        if (config.inMemoryCache) {
            if (this.isGroupCache()) {
                // @ts-ignore
                this.inMemoryCache = new InMemoryGroupCache_1.InMemoryGroupCache(config.inMemoryCache);
            }
            else {
                // @ts-ignore
                this.inMemoryCache = new memory_1.InMemoryCache(config.inMemoryCache);
            }
        }
        else {
            // @ts-ignore
            this.inMemoryCache = new memory_1.NoopCache();
        }
        this.asyncCache = config.asyncCache;
        this.logger = config.logger ?? Logger_1.defaultLogger;
        this.cacheUpdateErrorHandler = config.cacheUpdateErrorHandler ?? exports.DEFAULT_CACHE_ERROR_HANDLER;
        this.loadErrorHandler = config.loadErrorHandler ?? exports.DEFAULT_LOAD_ERROR_HANDLER;
        if (config.notificationConsumer) {
            if (!config.inMemoryCache) {
                throw new Error('Cannot set notificationConsumer when InMemoryCache is disabled');
            }
            this.notificationConsumer = config.notificationConsumer;
            this.notificationConsumer.setTargetCache(this.inMemoryCache);
            this.initPromises.push(this.notificationConsumer.subscribe().catch((err) => {
                /* c8 ignore next 1 */
                this.notificationConsumer.errorHandler(err, this.notificationConsumer.serverUuid, this.logger);
            }));
        }
        if (config.notificationPublisher) {
            this.notificationPublisher = config.notificationPublisher;
            this.initPromises.push(this.notificationPublisher.subscribe().catch((err) => {
                /* c8 ignore next 1 */
                this.notificationPublisher.errorHandler(err, this.notificationPublisher.channel, this.logger);
            }));
        }
        this.runningLoads = new Map();
    }
    async init() {
        await Promise.all(this.initPromises);
        this.initPromises = [];
    }
    async invalidateCache() {
        this.inMemoryCache.clear();
        if (this.asyncCache) {
            await this.asyncCache.clear().catch((err) => {
                this.cacheUpdateErrorHandler(err, undefined, this.asyncCache, this.logger);
            });
        }
        this.runningLoads.clear();
        if (this.notificationPublisher) {
            this.notificationPublisher.clear().catch((err) => {
                this.notificationPublisher.errorHandler(err, this.notificationPublisher.channel, this.logger);
            });
        }
    }
    async close() {
        if (this.asyncCache) {
            await this.asyncCache.close();
        }
        /* c8 ignore start */
        if (this.notificationConsumer) {
            try {
                await this.notificationConsumer.close();
            }
            catch (err) {
                // @ts-ignore
                this.logger.error(`Failed to close notification consumer: ${err.message}`);
            }
        }
        if (this.notificationPublisher) {
            try {
                await this.notificationPublisher.close();
            }
            catch (err) {
                // @ts-ignore
                this.logger.error(`Failed to close notification publisher: ${err.message}`);
            }
        }
        /* c8 ignore stop */
    }
}
exports.AbstractCache = AbstractCache;
//# sourceMappingURL=AbstractCache.js.map